-- Echo_IIR_1.vhd
-- Ian Clough 25 September 2007
-- UPDATED 5 October 2007
-- UPDATED 9 July 2010
-- UPDATED 9 September 2011
-- University of Queensland
-- Y(n) = a0*Xn + a1*Xn_1 - b1*Yn_1

library IEEE;
use IEEE.std_logic_1164.all;
use ieee.numeric_std.all;
entity Echo_IIR_1 is port
   (
       DATAIN      : in  std_logic_vector(11 downto 0);
       DATAOUT      : out  std_logic_vector(11 downto 0);
       SAMPLECLK    : in   std_logic
   );
end Echo_IIR_1;

architecture behavioral of Echo_IIR_1 is
--signal X : integer range -2048 to 2047 :=0;
--signal Y : integer range -2048 to 2047 :=0;
-- *********** Powers of 2 for Denom *****************
-- 32768	65536	131072	262144	524288	1048576
constant Denom: integer range 0 to 16777216:= 131072; 
constant k: integer range 1 to 256:=4; -- helps to prevent overflow
constant co_max: integer :=1048576;
constant co_min: integer :=0;
constant co_neg: integer := -co_max;
signal a0: integer range co_min to co_max-1;
signal a1: integer range co_min to co_max-1;
signal b0: integer range co_neg to co_max-1;
signal b1: integer range co_neg to co_max-1;
type registers is array (0 to 1) of integer range -2048 to 2047;
signal Yreg : registers;
signal Xreg : registers;
begin

 ECHO:process (SAMPLECLK)
    begin
--131072    131072   0     131072  -20760    0 -- DUMMY values

	  a0 <=524288/k; -- k =4, so 524288/k = 131072
	  a1 <= 0/k;
     b0 <= 0/k;
	  b1 <= 0/k;

     if falling_edge(SAMPLECLK) then
    Yreg(0) <= (a0*Xreg(0)/Denom + a1*Xreg(1)/Denom) - (b1*Yreg(1))/Denom; --calculate difference equation
	 Yreg(1) <= Yreg(0); --shift data
    Xreg(1) <= Xreg(0);
 
		end if;        --end fall
	 Xreg(0) <= to_INTEGER(unsigned(DATAIN))-2048 ;--change unipolar value (0 to 4095) to bipolar (-2048 to 2047)
   
    end process;
	  DATAOUT <=  STD_LOGIC_VECTOR(to_unsigned(Yreg(0)+2048,12));--convert bipolar to unipolar integer then to logic bit format
  end behavioral;


